export const baseUrl = import.meta.env.VITE_BASE_URL as string;

export async function apiAuth(
  path: string,
  data: any,
  method: string = "GET"
): Promise<any> {
  const req = new Request(`${baseUrl}api/${path}`, {
    method: method,
    headers: {
      "Content-Type": "application/json",
    },
    credentials: "include",
    ...(data && { body: JSON.stringify(data) }),
  });
  let resp: Response;
  try {
    resp = await fetch(req);
  } catch (e) {
    throw new Error(`request failed: ${e}`);
  }

  if (!resp.ok) {
    if (resp.status === 401) {
      logout();
      throw new Error("Unauthorized");
    }
  }
  return resp.json();
}

export type User = {
  username: string;
  full_name: string;
  email: string;
  player_id: number;
  scopes: string;
};

export async function currentUser(): Promise<User> {
  const req = new Request(`${baseUrl}api/users/me/`, {
    method: "GET",
    headers: {
      "Content-Type": "application/json",
    },
    credentials: "include",
  });
  let resp: Response;
  try {
    resp = await fetch(req);
  } catch (e) {
    throw new Error(`request failed: ${e}`);
  }

  if (!resp.ok) {
    if (resp.status === 401) {
      logout();
      throw new Error("Unauthorized");
    }
  }
  return resp.json() as Promise<User>;
}

export type LoginRequest = {
  username: string;
  password: string;
};

export const login = async (req: LoginRequest): Promise<void> => {
  try {
    const response = await fetch(`${baseUrl}api/token`, {
      method: "POST",
      headers: {
        "Content-Type": "application/x-www-form-urlencoded",
      },
      body: new URLSearchParams(req).toString(),
      credentials: "include",
    });
    if (!response.ok) {
      throw new Error(`HTTP error! status: ${response.status}`);
    }
  } catch (e) {
    console.error(e);
    throw e; // rethrow the error so it can be caught by the caller
  }
};

export const logout = async () => {
  try {
    await fetch(`${baseUrl}api/logout`, {
      method: "POST",
      credentials: "include",
    });
  } catch (e) {
    console.error(e);
  }
};
